@@ -54,6 +54,9 @@ chat_deps <- function() {
 #' @param fill Whether the chat element should try to vertically fill its
 #'   container, if the container is
 #'   [fillable](https://rstudio.github.io/bslib/articles/filling/index.html)
+#' @param icon_assistant The icon to use for the assistant chat messages.
+#'   Can be HTML or a tag in the form of [htmltools::HTML()] or
+#'   [htmltools::tags()]. If `None`, a default robot icon is used.
 #' @returns A Shiny tag object, suitable for inclusion in a Shiny UI
 #'
 #' @examplesIf interactive()
@@ -67,7 +70,7 @@ chat_deps <- function() {
 #'
 #' server <- function(input, output, session) {
 #'   observeEvent(input$chat_user_input, {
-#'     # In a real app, this would call out to a chat model or API,
+#'     # In a real app, this would call out to a chat client or API,
 #'     # perhaps using the 'ellmer' package.
 #'     response <- paste0(
 #'       "You said:\n\n",
@@ -76,6 +79,7 @@ chat_deps <- function() {
 #'       "</blockquote>"
 #'     )
 #'     chat_append("chat", response)
+#'     chat_append("chat", stream)
 #'   })
 #' }
 #'
@@ -89,7 +93,8 @@ chat_ui <- function(
   placeholder = "Enter a message...",
   width = "min(680px, 100%)",
   height = "auto",
-  fill = TRUE
+  fill = TRUE,
+  icon_assistant = NULL
 ) {
   attrs <- rlang::list2(...)
   if (!all(nzchar(rlang::names2(attrs)))) {
@@ -104,12 +109,6 @@ chat_ui <- function(
       role <- x[["role"]] %||% role
     }
 
-    if (isTRUE(role == "user")) {
-      tag_name <- "shiny-user-message"
-    } else {
-      tag_name <- "shiny-chat-message"
-    }
-
     # `content` is most likely a string, so avoid overhead in that case
     # (it's also important that we *don't escape HTML* here).
     if (is.character(content)) {
@@ -119,9 +118,11 @@ chat_ui <- function(
     }
 
     tag(
-      tag_name,
+      "shiny-chat-message",
       rlang::list2(
+        `data-role` = role,
         content = ui[["html"]],
+        icon = if (!is.null(icon_assistant)) as.character(icon_assistant),
         ui[["dependencies"]],
       )
     )
@@ -137,13 +138,19 @@ chat_ui <- function(
       ),
       placeholder = placeholder,
       fill = if (isTRUE(fill)) NA else NULL,
+      # Also include icon on the parent so that when messages are dynamically added,
+      # we know the default icon has changed
+      `icon-assistant` = if (!is.null(icon_assistant)) {
+        as.character(icon_assistant)
+      },
       ...,
       tag("shiny-chat-messages", message_tags),
       tag(
         "shiny-chat-input",
         list(id = paste0(id, "_user_input"), placeholder = placeholder)
       ),
-      chat_deps()
+      chat_deps(),
+      htmltools::findDependencies(icon_assistant)
     )
   )
 
@@ -163,7 +170,7 @@ chat_ui <- function(
 #' `chat_async`, and `stream_async` methods, respectively).
 #'
 #' This function should be called from a Shiny app's server. It is generally
-#' used to append the model's response to the chat, while user messages are
+#' used to append the client's response to the chat, while user messages are
 #' added to the chat UI automatically by the front-end. You'd only need to use
 #' `chat_append(role="user")` if you are programmatically generating queries
 #' from the server and sending them on behalf of the user, and want them to be
@@ -175,9 +182,8 @@ chat_ui <- function(
 #' an error occurs while producing the message (e.g., an iteration in
 #' `stream_async` fails), the promise returned by `chat_append` will reject with
 #' the error. If the `chat_append` call is the last expression in a Shiny
-#' observer, Shiny will see that the observer failed, and end the user session.
-#' If you prefer to handle the error gracefully, use [promises::catch()] on the
-#' promise returned by `chat_append`.
+#' observer, shinychat will log the error message and show a message that the
+#' error occurred in the chat UI.
 #'
 #' @param id The ID of the chat element
 #' @param response The message or message stream to append to the chat element.
@@ -194,6 +200,9 @@ chat_ui <- function(
 #'
 #' @param role The role of the message (either "assistant" or "user"). Defaults
 #'   to "assistant".
+#' @param icon An optional icon to display next to the message, currently only
+#'   used for assistant messages. The icon can be any HTML element (e.g., an
+#'   [htmltools::img()] tag) or a string of HTML.
 #' @param session The Shiny session object
 #'
 #' @returns Returns a promise that resolves to the contents of the stream, or an
@@ -245,13 +254,14 @@ chat_append <- function(
   id,
   response,
   role = c("assistant", "user"),
+  icon = NULL,
   session = getDefaultReactiveDomain()
 ) {
   check_active_session(session)
   role <- match.arg(role)
 
   stream <- as_generator(response)
-  chat_append_stream(id, stream, role = role, session = session)
+  chat_append_stream(id, stream, role = role, icon = icon, session = session)
 }
 
 #' Low-level function to append a message to a chat control
@@ -273,6 +283,9 @@ chat_append <- function(
 #'   then the new content is appended to the existing message content. If
 #'   `"replace"`, then the existing message content is replaced by the new
 #'   content. Ignored if `chunk` is `FALSE`.
+#' @param icon An optional icon to display next to the message, currently only
+#'   used for assistant messages. The icon can be any HTML element (e.g.,
+#'   [htmltools::img()] tag) or a string of HTML.
 #' @param session The Shiny session object
 #'
 #' @returns Returns nothing (\code{invisible(NULL)}).
@@ -328,12 +341,13 @@ chat_append_message <- function(
   msg,
   chunk = TRUE,
   operation = c("append", "replace"),
+  icon = NULL,
   session = getDefaultReactiveDomain()
 ) {
   check_active_session(session)
 
   if (!is.list(msg)) {
-    rlang::abort("msg must be a named list with 'role' and 'content' fields")
+    rlang::abort("`msg` must be a named list with 'role' and 'content' fields")
   }
   if (!isTRUE(msg[["role"]] %in% c("user", "assistant"))) {
     warning("Invalid role argument; must be 'user' or 'assistant'")
@@ -359,7 +373,13 @@ chat_append_message <- function(
   content <- msg[["content"]]
   is_html <- inherits(
     content,
-    c("shiny.tag", "shiny.tag.list", "html", "htmlwidget")
+    c(
+      "shiny.tag",
+      "shiny.tag.list",
+      "html",
+      "htmlwidget",
+      "shinychat_tool_card"
+    )
   )
   content_type <- if (is_html) "html" else "markdown"
 
@@ -379,8 +399,18 @@ chat_append_message <- function(
     ui <- process_ui(content, session)
   }
 
+  msg_content <- ui[["html"]]
+  if (is_html) {
+    # Code blocks with `{=html}` infostrings are rendered as-is by a custom
+    # rendering method in markdown-stream.ts
+    msg_content <- sprintf(
+      "\n\n````````{=html}\n%s\n````````\n\n",
+      msg_content
+    )
+  }
+
   msg <- list(
-    content = ui[["html"]],
+    content = msg_content,
     role = msg[["role"]],
     content_type = content_type,
     html_deps = ui[["deps"]],
@@ -388,6 +418,10 @@ chat_append_message <- function(
     operation = operation
   )
 
+  if (!is.null(icon)) {
+    msg$icon <- as.character(icon)
+  }
+
   session$sendCustomMessage(
     "shinyChatMessage",
     list(
@@ -404,9 +438,11 @@ chat_append_stream <- function(
   id,
   stream,
   role = "assistant",
+  icon = NULL,
   session = getDefaultReactiveDomain()
 ) {
-  result <- chat_append_stream_impl(id, stream, role, session)
+  result <- chat_append_stream_impl(id, stream, role, icon, session)
+  result <- chat_update_bookmark(id, result, session = session)
   # Handle erroneous result...
   result <- promises::catch(result, function(reason) {
     # ...but rethrow the error as a silent error, so the caller can also handle
@@ -451,14 +487,21 @@ rlang::on_load(
     id,
     stream,
     role = "assistant",
+    icon = NULL,
     session = shiny::getDefaultReactiveDomain()
   ) {
-    chat_append_message(
-      id,
-      list(role = role, content = ""),
-      chunk = "start",
-      session = session
-    )
+    chat_append_ <- function(content, chunk = TRUE, ...) {
+      chat_append_message(
+        id,
+        msg = list(role = role, content = content),
+        operation = "append",
+        chunk = chunk,
+        session = session,
+        ...
+      )
+    }
+
+    chat_append_("", chunk = "start", icon = icon)
 
     res <- fastmap::fastqueue(200)
 
@@ -472,22 +515,20 @@ rlang::on_load(
 
       res$add(msg)
 
-      chat_append_message(
-        id,
-        list(role = role, content = msg),
-        chunk = TRUE,
-        operation = "append",
-        session = session
-      )
+      if (S7::S7_inherits(msg, ellmer::ContentToolResult)) {
+        if (!is.null(msg@request)) {
+          session$sendCustomMessage("shiny-tool-request-hide", msg@request@id)
+        }
+      }
+
+      if (S7::S7_inherits(msg, ellmer::Content)) {
+        msg <- contents_shinychat(msg)
+      }
+
+      chat_append_(msg)
     }
 
-    chat_append_message(
-      id,
-      list(role = role, content = ""),
-      chunk = "end",
-      operation = "append",
-      session = session
-    )
+    chat_append_("", chunk = "end")
 
     res <- res$as_list()
     if (every(res, is.character)) {
@@ -539,3 +580,83 @@ chat_clear <- function(id, session = getDefaultReactiveDomain()) {
     )
   )
 }
+
+
+#' Update the user input of a chat control
+#'
+#' @param id The ID of the chat element
+#' @param ... Currently unused, but reserved for future use.
+#' @param value The value to set the user input to. If `NULL`, the input will not be updated.
+#' @param placeholder The placeholder text for the user input
+#' @param submit Whether to automatically submit the text for the user. Requires `value`.
+#' @param focus Whether to move focus to the input element. Requires `value`.
+#' @param session The Shiny session object
+#'
+#' @export
+#' @examplesIf interactive()
+#' library(shiny)
+#' library(bslib)
+#' library(shinychat)
+#'
+#' ui <- page_fillable(
+#'   chat_ui("chat"),
+#'   layout_columns(
+#'     fill = FALSE,
+#'     actionButton("update_placeholder", "Update placeholder"),
+#'     actionButton("update_value", "Update user input")
+#'   )
+#' )
+#'
+#' server <- function(input, output, session) {
+#'   observeEvent(input$update_placeholder, {
+#'     update_chat_user_input("chat", placeholder = "New placeholder text")
+#'   })
+#'
+#'   observeEvent(input$update_value, {
+#'     update_chat_user_input("chat", value = "New user input", focus = TRUE)
+#'   })
+#'
+#'   observeEvent(input$chat_user_input, {
+#'     response <- paste0("You said: ", input$chat_user_input)
+#'     chat_append("chat", response)
+#'   })
+#' }
+#'
+#' shinyApp(ui, server)
+
+update_chat_user_input <- function(
+  id,
+  ...,
+  value = NULL,
+  placeholder = NULL,
+  submit = FALSE,
+  focus = FALSE,
+  session = getDefaultReactiveDomain()
+) {
+  rlang::check_dots_empty()
+  check_active_session(session)
+
+  if (is.null(value) && (submit || focus)) {
+    rlang::abort(
+      "An input `value` must be provided when `submit` or `focus` are `TRUE`."
+    )
+  }
+
+  vals <- drop_nulls(
+    list(
+      value = value,
+      placeholder = placeholder,
+      submit = submit,
+      focus = focus
+    )
+  )
+
+  session$sendCustomMessage(
+    "shinyChatMessage",
+    list(
+      id = resolve_id(id, session),
+      handler = "shiny-chat-update-user-input",
+      obj = vals
+    )
+  )
+}
